@@ -63,13 +63,18 @@ double *inla_cgeneric_iid_model(inla_cgeneric_cmd_tp cmd, double *theta, inla_cg
 	{
 		// return a vector of indices with format
 		// c(N, M, ii, jj)
-		// where ii<=jj and both ii and jj are non-decreasing
-		// and M is the length of ii
+		// where ii<=jj, ii is non-decreasing and jj is non-decreasing for the same ii
+		// so like the loop
+		// for i=0, ...
+		// for j=i, ...
+		// G_ij = 
+		// and M is the total length while N is the dimension
 
+		int M = N;
 		ret = Calloc(2 + 2 * N, double);
 		ret[0] = N;				       /* dimension */
-		ret[1] = N;				       /* number of (i <= j) */
-		for (int i = 0; i < N; i++) {
+		ret[1] = M;				       /* number of (i <= j) */
+		for (int i = 0; i < M; i++) {
 			ret[2 + i] = i;			       /* i */
 			ret[2 + N + i] = i;		       /* j */
 		}
@@ -78,29 +83,13 @@ double *inla_cgeneric_iid_model(inla_cgeneric_cmd_tp cmd, double *theta, inla_cg
 
 	case INLA_CGENERIC_Q:
 	{
-		if (1) {
-			// optimized format
-			// return c(N, M, Qij) in the same order as defined in INLA_CGENERIC_GRAPH
-			// where M is the length of Qij
-			ret = Calloc(2 + N, double);
-			ret[0] = -1;			       /* code for optimized output */
-			ret[1] = N;			       /* number of (i <= j) */
-			for (int i = 0; i < N; i++) {
-				ret[2 + i] = prec;
-			}
-		} else {
-			// plain format, but the optimized format above is better to use
-			// return c(N, M, ii, jj, Qij)
-			// where ii<=jj and both ii and jj are non-decreasing
-			// and M is the length of ii
-			ret = Calloc(2 + 3 * N, double);
-			ret[0] = N;
-			ret[1] = N;
-			for (int i = 0; i < N; i++) {
-				ret[2 + i] = i;		       /* i */
-				ret[2 + N + i] = i;	       /* j */
-				ret[2 + 2 * N + i] = prec;     /* Q_ij */
-			}
+		// return c(-1, M, Qij) in the same order as defined in INLA_CGENERIC_GRAPH
+		int M = N;
+		ret = Calloc(2 + N, double);
+		ret[0] = -1;				       /* REQUIRED! */
+		ret[1] = M;				       /* number of (i <= j) */
+		for (int i = 0; i < M; i++) {
+			ret[2 + i] = prec;
 		}
 		break;
 	}
@@ -178,21 +167,25 @@ double *inla_cgeneric_ar1_model(inla_cgeneric_cmd_tp cmd, double *theta, inla_cg
 	{
 		// return a vector of indices with format
 		// c(N, M, ii, jj)
-		// where ii<=jj and both ii and jj are non-decreasing
+		// where ii<=jj, ii is non-decreasing and jj is non-decreasing for the same ii
+		// so like the loop
+		// for i=0, ...
+		// for j=i, ...
+		// G_ij = 
 		// and M is the length of ii
 
-		int m = N + N - 1, offset, i, k;
-		ret = Calloc(2 + 2 * m, double);
+		int M = N + N - 1, offset, i, k;
+		ret = Calloc(2 + 2 * M, double);
 
 		offset = 2;
 		ret[0] = N;				       /* dimension */
-		ret[1] = m;				       /* number of (i <= j) */
+		ret[1] = M;				       /* number of (i <= j) */
 		for (k = i = 0; i < N; i++) {
 			ret[offset + k] = i;		       /* i */
-			ret[offset + m + k++] = i;	       /* j */
+			ret[offset + M + k++] = i;	       /* j */
 			if (i < N - 1) {
 				ret[offset + k] = i;	       /* i */
-				ret[offset + m + k++] = i + 1; /* j */
+				ret[offset + M + k++] = i + 1; /* j */
 			}
 		}
 		break;
@@ -201,17 +194,17 @@ double *inla_cgeneric_ar1_model(inla_cgeneric_cmd_tp cmd, double *theta, inla_cg
 	case INLA_CGENERIC_Q:
 	{
 		// optimized format
-		// return c(N, M, Qij) in the same order as defined in INLA_CGENERIC_GRAPH
+		// return c(-1, M, Qij) in the same order as defined in INLA_CGENERIC_GRAPH
 		// where M is the length of Qij
 
 		double param = prec / (1.0 - SQR(rho));
-		int m = N + N - 1;
+		int M = N + N - 1;
 		int offset, i, k;
-		ret = Calloc(2 + m, double);
+		ret = Calloc(2 + M, double);
 
 		offset = 2;
-		ret[0] = -1;
-		ret[1] = m;
+		ret[0] = -1;				       /* REQUIRED */
+		ret[1] = M;
 		for (i = k = 0; i < N; i++) {
 			ret[offset + k++] = param * (i == 0 || i == N - 1 ? 1.0 : (1.0 + SQR(rho)));
 			if (i < N - 1) {
@@ -269,3 +262,128 @@ double *inla_cgeneric_ar1_model(inla_cgeneric_cmd_tp cmd, double *theta, inla_cg
 
 	return (ret);
 }
+
+double *inla_cgeneric_generic0_model(inla_cgeneric_cmd_tp cmd, double *theta, inla_cgeneric_data_tp * data)
+{
+	double *ret = NULL, prec, lprec;
+
+	if (theta) {
+		lprec = theta[0];
+		prec = exp(lprec);
+	} else {
+		prec = lprec = NAN;
+	}
+
+	assert(data->n_ints > 0);
+	assert(!strcasecmp(data->ints[0]->name, "n"));	       // this will always be the case
+	int N = data->ints[0]->ints[0];			       // this will always be the case
+	assert(N > 0);
+
+	/*
+	 * we assume Cmatrix is stored column-wise, like
+	 * for(i...)
+	 *     for(j=i...)
+	 *         Q_ij
+	 * 
+	 * so, upper half only
+	 */
+	assert(data->n_smat > 0);
+	assert(!strcasecmp(data->smats[0]->name, "Cmatrix"));
+	inla_cgeneric_smat_tp *Cmatrix = data->smats[0];
+	assert(N == Cmatrix->nrow);
+	assert(N == Cmatrix->ncol);
+
+	switch (cmd) {
+	case INLA_CGENERIC_VOID:
+	{
+		assert(!(cmd == INLA_CGENERIC_VOID));
+		break;
+	}
+
+	case INLA_CGENERIC_GRAPH:
+	{
+		// return a vector of indices with format
+		// c(N, M, ii, jj)
+		// where ii<=jj, ii is non-decreasing and jj is non-decreasing for the same ii
+		// so like the loop
+		// for i=0, ...
+		// for j=i, ...
+		// G_ij = 
+		// and M is the length of ii
+
+		int M = Cmatrix->n, offset;
+		ret = Calloc(2 + 2 * M, double);
+		offset = 2;
+		ret[0] = N;				       /* dimension */
+		ret[1] = M;				       /* number of (i <= j) */
+		for (int k = 0; k < M; k++) {
+			ret[offset + k] = Cmatrix->i[k];       /* i */
+			ret[offset + M + k] = Cmatrix->j[k];   /* j */
+		}
+		break;
+	}
+
+	case INLA_CGENERIC_Q:
+	{
+		// optimized format
+		// return c(-1, M, Qij) in the same order as defined in INLA_CGENERIC_GRAPH
+		// where M is the length of Qij
+
+		int M = Cmatrix->n;
+		int offset;
+		ret = Calloc(2 + M, double);
+
+		offset = 2;
+		ret[0] = -1;				       /* REQUIRED */
+		ret[1] = M;
+		for (int k = 0; k < M; k++) {
+			ret[offset + k] = prec * Cmatrix->x[k];
+		}
+		break;
+	}
+
+	case INLA_CGENERIC_MU:
+	{
+		// return (N, mu)
+		// if N==0 then mu is not needed as its taken to be mu[]==0
+
+		ret = Calloc(1, double);
+		ret[0] = 0;
+		break;
+	}
+
+	case INLA_CGENERIC_INITIAL:
+	{
+		// return c(M, initials)
+		// where M is the number of hyperparameters
+
+		ret = Calloc(2, double);
+		ret[0] = 1;
+		ret[1] = 4.0;
+		break;
+	}
+
+	case INLA_CGENERIC_LOG_NORM_CONST:
+	{
+		// return c(NORM_CONST) or a NULL-pointer if INLA should compute it by itself
+		ret = Calloc(1, double);
+		ret[0] = N / 2.0 * (lprec - log(2.0 * M_PI));
+		break;
+	}
+
+	case INLA_CGENERIC_LOG_PRIOR:
+	{
+		// return c(LOG_PRIOR). with a Gamma(1,1) for precision, this is the log prior for the log(precision).
+
+		ret = Calloc(1, double);
+		ret[0] = -prec + lprec;
+		break;
+	}
+
+	case INLA_CGENERIC_QUIT:
+	default:
+		break;
+	}
+
+	return (ret);
+}
