@@ -3,6 +3,7 @@ library(INLA)
 if (file.exists("myinit.R")) source("myinit.R")
 inla.setOption(smtp='taucs')
 inla.setOption(num.threads="1:1")
+inla.setOption(inla.mode="experimental")
 knitr::opts_chunk$set(fig.path="figures/old-faq/")
 set.seed(123)
 source("render_toc.R")
@@ -145,6 +146,7 @@ link[n2  + which(is.na(y[1:n2 + n2]))] = 2
 
 r = inla(Y ~ 1 + x,  family = c("poisson", "nbinomial"),
         data = list(Y=Y, x=x),
+		control.family= list(list(), list()),
         control.predictor = list(link = link))
 plot(exp(eta),type ="l")
 points(r$summary.fitted.values$mean, pch=19)
@@ -337,6 +339,7 @@ lcs = inla.make.lincombs(Predictor = B)
 r = inla(formula,  data = data.frame(Y, time),
         control.predictor = list(compute=TRUE),
         lincomb = lcs,
+		inla.mode="classic",
         control.inla = list(lincomb.derived.correlation.matrix=TRUE))
 
 print(round(r$misc$lincomb.derived.correlation.matrix,dig=3))
@@ -546,6 +549,7 @@ formula = Y ~ X - 1
 ## `family' is now
 result = inla(formula,
         family = c("poisson", "binomial"),
+		control.family=list(list(), list()),
         data = list(Y=Y, X=X),
         E = E, Ntrials = Ntrials)
 summary(result)
@@ -596,13 +600,13 @@ formula = y ~ f(i, model="ar1", replicate=r, constr=TRUE) + rf -1
 data = list(y=y, i=i, r=r, rf=rf)
 
 ## parameters for the binomial and the poisson
-Ntrial = rep(NA, 3*n)
-Ntrial[1:n] = Nt
+Ntrials = rep(NA, 3*n)
+Ntrials[1:n] = Nt
 E = rep(NA, 3*n)
 E[1:n + n] = Ep
 
 result = inla(formula, family = c("binomial", "poisson", "normal"),
-              data = data, Ntrial = Ntrial, E = E,
+              data = data, Ntrials = Ntrials, E = E,
               control.family = list(
                       list(),
                       list(),
